@@ -7,6 +7,7 @@ from os import path as _path
 
 from strcuta import otoini as _otoini
 from strcuta import prefixmap as _prefixmap
+from strcuta import cursor as _cursor
 from strcuta import frq as _frq
 from strcuta import wav as _wav
 
@@ -29,9 +30,6 @@ class Counts:
         self.vow = full - con
         self.stretchable = self.vow
 
-def _ms2nframes(rate, millisec):
-    return round(rate * millisec / 1000)
-
 class Type:
     def __init__(self, rootdir, otos, prefixes):
         self.rootdir = rootdir
@@ -46,22 +44,29 @@ class Type:
 
     def voice(self, spell, key):
         info = self.oto(spell, key)
-        w = _wav.load(_path.join(self.rootdir, info.source))
+        wav_path = _path.join(self.rootdir, info.source)
+        w = _wav.load(wav_path)
 
         rate = w.parameter.framerate
         nframes = w.parameter.nframes
 
-        nf_offset = _ms2nframes(rate, info.offset)
-        nf_con = _ms2nframes(rate, info.consonant)
-        nf_pre = _ms2nframes(rate, info.preutterance)
-        nf_ovl = _ms2nframes(rate, info.overlap)
+        nf_offset = _cursor.ms(info.offset)._resolve(rate, 1)
+        nf_con = _cursor.ms(info.consonant)._resolve(rate, 1)
+        nf_pre = _cursor.ms(info.preutterance)._resolve(rate, 1)
+        nf_ovl = _cursor.ms(info.overlap)._resolve(rate, 1)
         if info.duration != None:
-            nf_used = _ms2nframes(rate, info.duration)
+            nf_used = _cursor.ms(info.duration)._resolve(rate, 1)
         else:
-            nf_cutoff = _ms2nframes(rate, info.cutoff)
+            nf_cutoff = _cursor.ms(info.cutoff)._resolve(rate, 1)
             nf_used = nframes - nf_offset - nf_cutoff
         
         return Voice(
+                _frq=_frq_memoized(
+                    wav_path[:-4] + "_wav.frq",
+                    nf_offset,
+                    nf_used,
+                    w
+                    ),
                 wave=w[nf_offset : nf_offset + nf_used],
                 count=Counts(
                     pre=nf_pre,
@@ -71,8 +76,20 @@ class Type:
                     )
                 )
 
+def _frq_memoized(path, offset, used, wave):
+    memo = None
+    def closure():
+        nonlocal memo
+        if memo == None:
+            f = _frq.load(path, wave)
+            memo = f[round(offset / f.sample_interval) : round((offset + used) / f.sample_interval)]
+        return memo
+    return closure
+
+
 class Voice(_wav.Type):
-    def __init__(self, wave, count):
+    def __init__(self, _frq, wave, count):
+        self._frq = _frq
         self.count = count
         self.cursor = Cursors(
                 start=0,
@@ -85,6 +102,9 @@ class Voice(_wav.Type):
                 parameter=wave.parameter,
                 frames=wave.frames)
 
+    def frq(self):
+        return self._frq()
+
     def range_pre(self):
         return slice(self.cursor.start, self.cursor.end_pre)
 
