@@ -4,6 +4,7 @@
 # https://www.boost.org/LICENSE_1_0.txt)
 
 import wave as _wave
+from strcuta import cursor as _cursor
 
 class Type:
     def __init__(self, parameter, frames):
@@ -11,16 +12,18 @@ class Type:
         self.parameter = parameter
         self.frames = frames
 
+    def __len__(self):
+        return self.parameter.nframes
+
     def __getitem__(self, key):
         if isinstance(key, slice):
-            k = slice(key.start * self.parameter.sampwidth, key.stop * self.parameter.sampwidth)
+            k = _cursor.resolve_slice(key, self.parameter.framerate, self.parameter.sampwidth)
+            f = self.frames[k]
             return Type(
-                    parameter=self.parameter._replace(nframes=k.stop - k.start),
-                    frames=self.frames[k])
-        elif isinstance(key, int):
-            return self.frames[key * self.parameter.sampwidth]
+                    parameter=self.parameter._replace(nframes=int(len(f) / self.parameter.sampwidth)),
+                    frames=f)
         else:
-            raise "not an index"
+            return self.frames[_cursor.resolve_to_slice(key, self.parameter.framerate, self.parameter.sampwidth)]
 
     def write(self, outputpath):
         with _wave.open(outputpath, mode="wb") as w:
